{% macro add_button_style() %}
    <style type="text/css">
        .button {
            background-color: white;
            color: black;
            padding: 16px 32px;
            text-align: center;
            text-decoration: none;
            display: inline-block;
            font-size: 16px;
            margin: 15px;
            -webkit-transition-duration: 0.4s;
            transition-duration: 0.4s;
            cursor: pointer;
        }
        .button:hover {
            color: white;
        }
        .default-btn {
            border: 2px solid #f44336;
        }
        .default-btn:hover {
            background-color: #f44336;
        }
        .request-btn {
            border: 2px solid #36a2f4;
        }
        .request-btn:hover {
            background-color: #36a2f4;
        }
        .edit-btn {
            border: 2px solid #020924;
        }
        .edit-btn:hover {
            background-color: #020924;
        }
        .link-button {
            background-color: white;
            color: black;
            padding: 2px 8px;
            text-align: center;
            text-decoration: none;
            display: inline-block;
            font-size: 14px;
            margin: 2px;
            -webkit-transition-duration: 0.4s;
            transition-duration: 0.4s;
            cursor: pointer;
            width: 140px;
        }
        .link-button:hover {
            color: white;
        }
        .success-btn {
            border: 2px solid #33cc33;
        }
        .success-btn:hover {
            background-color: #33cc33;
        }
        form {
            display: inline-block;
        }
    </style>
{% endmacro %}
