class FeatureTypeExtractionError(RuntimeError):
    """Exception for feature type extraction error."""


class ScenariosTestFileNotFound(FileNotFoundError):
    """Exception for situation without scenarios test files."""
